package cn.com.qmth.examcloud.dengbao.utils;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.StringUtils;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

public class AESEncryptor {

    private static final Logger log = LoggerFactory.getLogger(AESEncryptor.class);

    private static final String AES = "AES";

    public String encrypt(String key, final Object plaintext) {
        if (null == plaintext) {
            return null;
        }

        try {
            byte[] data = StringUtils.getBytesUtf8(String.valueOf(plaintext));
            byte[] result = getCipher(key, Cipher.ENCRYPT_MODE).doFinal(data);
            return Base64.encodeBase64String(result);
        } catch (Exception e) {
            log.error(e.getMessage());
            throw new RuntimeException(e);
        }
    }

    public String decrypt(String key, final String ciphertext) {
        if (null == ciphertext) {
            return null;
        }

        try {
            byte[] data = Base64.decodeBase64(ciphertext);
            byte[] result = getCipher(key, Cipher.DECRYPT_MODE).doFinal(data);
            return new String(result, StandardCharsets.UTF_8);
        } catch (Exception e) {
            log.error(e.getMessage());
            throw new RuntimeException("Decrypt fail, maybe secretKey is wrong...", e);
        }
    }

    private Cipher getCipher(String key, final int decryptMode)
            throws NoSuchPaddingException, NoSuchAlgorithmException, InvalidKeyException {
        Cipher result = Cipher.getInstance(AES);
        result.init(decryptMode, new SecretKeySpec(createSecretKey(key), AES));
        return result;
    }

    private byte[] createSecretKey(String key) {
        return Arrays.copyOf(DigestUtils.sha1(key), 16);
    }

}
